################################################################################################
##################    BOOTSTRAP OF THE GRADE EUQATIONS AND PREFERENCE ##########################
################################################################################################
# libraries
library(nlme) 
library(foreign)
library(stats)
library(nloptr)
library(splines)
library(doParallel)
cl <- makeCluster(24)  # set the number of cpu you want to use
registerDoParallel(cl)

setwd("~/R/CMX2018QE/Bootstrap")
# sources of subroutines before bootstraping counterfactuals
source("CMX_Proc_FUNboot.R")
source("CMX_Proc_BOOTALL.R")
source("CMX_Proc_Sucprob.R")
source("CMX_Proc_LikBOOT.R")
source("CMX_Proc_Choice.R")
source("CMX_Proc_Resim.R")
# load the functions for bootstraping counterfactuals
source("CMX_Proc_FUNcountsys.R")
source("CMX_Proc_ARCboot.R")
source("CMX_Proc_TCboot.R")
source("CMX_Proc_TMboot.R")

# source of data and parameters
set.seed(1894285437, kind = NULL)
#  set the numbers for the bootstrap
nboot <- 499     #  number of bootraps
nsimT <- 100     #  number of system resimulations in each bootstrap
nsim  <- 100     #  number of simulations in the success probability computation

################################################################################################
#####   Parameters and vectors used for bootstrap ##############################################
# PARgrade : all the parameter from the grade equation estimation 
#     PARgrade[[1]]: the parameters of the first grade equation (vector of size 12)
#     PARgrade[[2]]: the parameters of the second grade equation (vector of size 9)
#     PARgrade[[3]]: Control function parameter in the ranking combination (a number)
#
# ubootsamp: the empirical residuals of grade equations, choices and Enem scores
#     ubootsamp[[1]]:  u1 hat, the residuals of the first grade equation (vector of size nobs)
#     ubootsamp[[2]]:  u2 hat, the residuals of the second grade equation (vector of size nobsm2)
#     ubootsamp[[3]]:  the observed choice, 1 means choosing Sobral (vector of size nobs)
#     ubootsamp[[4]]:  the observed Enem scores (vector of size nobs)
#
# datagrade1: all the covariates of the first grade equation  (matrix of size nobs*11)
# datagrade2: all the covariates of the second grade equation (matrix of size nobs*7)
#
# PARpref: the parameters of the preference estimation and the boundaries for bootstraps
#     PARpref[[1]]: the paramters of the preference estimation (vector of size 11)
#     PARpref[[2]]: the upper bound of parameters (vector of size 11)
#     PARpref[[3]]: the lower bound of parameters (vector of size 11)
#
# vecpref: some results from the preference estimation 
#     vecpref[[1]]: log utility difference between Sobral and Fortaleza (vector of size nobs)
#     vecpref[[2]]: the proabailities of being one particular type of students: Deltas (matrix nobs*3)
#     vecpref[[3]]: the expected success probabilities in the original system at both schools (matrix nobs*2)
#
# datapref: all the covariates of the preference estimation (matrix of size nobs*6)
################################################################################################
# load the results from the previous estimation
load("CMX_Prep_boot.rdata")

PARgrade   <- prep_boot[[1]]
ubootsamp  <- prep_boot[[2]]
datagrade1 <- prep_boot[[3]]
datagrade2 <- prep_boot[[4]]
PARpref    <- prep_boot[[5]]
vecpref    <- prep_boot[[6]]
datapref   <- prep_boot[[7]]

# number of observations and number of final seats
nobs<- length(vecpref[,1])
nos <- 40
nof <- 150 

#  random draw from uniform distribution 
simul <- matrix(runif(nobs*nsim), nrow = nobs)     

# the bootstrap results matrix
m <- matrix(rep(0),nrow = (length(c(PARgrade[[1]], PARgrade[[2]],PARpref[[1]])) + 8) , ncol=nboot)
s <- floor(423718954*runif(nboot))    # set seed for random draws

# bootstraping the grades and preference estimates 
  m <- foreach(b = (1:nboot), .packages = c("foreign","nlme","splines","stats", "nloptr"),.combine="cbind") %dopar%
{
  FUNboot(PARgrade,PARpref,vecpref,ubootsamp,datagrade1,datagrade2,datapref, s[b])  
}

# results outputs
BootRES<-cbind(c(PARgrade[[1]],PARgrade[[2]],PARpref[[1]],183,190,239,244,1,2,3,4),apply(m,1,mean),apply(m,1,sd))
rownames(BootRES)[22:40]<-c("delta0S","deltam0S","delta0SF","deltam0SF","intercept","m0p","femalep","agep","repetp",
                                "educfp","educmp","t1s","t1f","t2s","t2f","t1ssd","t1fsd","t2ssd","t2fsd")
rownames(BootRES)[1:length(PARgrade[[1]])]<-names(PARgrade[[1]])
rownames(BootRES)[(length(PARgrade[[1]])+1):21]<-names(PARgrade[[2]])
colnames(BootRES)<-c("original system","bootstrap mean","bootstrap sd")

# save the bootstrap results
save(BootRES, file="CMX_Bootstrap_Results.rdata")
save(m, file="CMX_Bootstrap_Matrix.rdata")

stopCluster(cl)



