###########################################################################################
###########  Admission rate change counterfactual #########################################
##########################################################################################
# load the functions necessary 
source("CMX_Proc_Choice.R")
source("CMX_Proc_FUNcountsys.R")
source("CMX_Proc_Resim.R")
library(doParallel)
cl <- makeCluster(24)  # set the number of cpu you want to use
registerDoParallel(cl)
###################################################################################################
#####   Parameters and vectors used for counterfactual simulations ################################
# PARgrade : all the parameter from the grade equation estimation 
#     PARgrade[[1]]: the parameters of the first grade equation (vector of size 12)
#     PARgrade[[2]]: the parameters of the second grade equation (vector of size 9)
#     PARgrade[[3]]: Control function parameter in the ranking combination (a number)
#
# ubootsamp: the empirical residuals of grade equations, choices and Enem scores
#     ubootsamp[[1]]:  u1 hat, the residuals of the first grade equation (vector of size nobs)
#     ubootsamp[[2]]:  u2 hat, the residuals of the second grade equation (vector of size nobsm2)
#     ubootsamp[[3]]:  the observed choice, 1 means choosing Sobral (vector of size nobs)
#     ubootsamp[[4]]:  the observed Enem scores (vector of size nobs)
#
# datagrade1: all the covariates of the first grade equation  (matrix of size nobs*11)
# datagrade2: all the covariates of the second grade equation (matrix of size nobs*7)
#
# PARpref: the parameters of the preference estimation and the boundaries for bootstraps
#     PARpref[[1]]: the paramters of the preference estimation (vector of size 11)
#     PARpref[[2]]: the upper bound of parameters (vector of size 11)
#     PARpref[[3]]: the lower bound of parameters (vector of size 11)
#
# vecpref: some results from the preference estimation 
#     vecpref[[1]]: log utility difference between Sobral and Fortaleza (vector of size nobs)
#     vecpref[[2]]: the proabailities of being one particular type of students: Deltas (matrix nobs*3)
#     vecpref[[3]]: the expected success probabilities in the original system at both schools (matrix nobs*2)
#
# datapref: all the covariates of the preference estimation (matrix of size nobs*6)
################################################################################################
 

set.seed(1894285437, kind = NULL)

load("CMX_Prep_boot.rdata")
PARgrade   <- prep_boot[[1]]
ubootsamp  <- prep_boot[[2]]
datagrade1 <- prep_boot[[3]]
datagrade2 <- prep_boot[[4]]
PARpref    <- prep_boot[[5]]
vecpref    <- prep_boot[[6]]
datapref   <- prep_boot[[7]]
mft        <- prep_boot[[8]]

# number of observations and number of final seats
nobs<- length(vecpref[,1])
nos <- 40
nof <- 150 

nsimT <- 499     #  number of system resimulations in each bootstrap
nsim  <- 999     #  number of simulations in the success probability computation
nresim <- 4999
#  random draw from uniform distribution for the success probability computation
simul <- matrix(runif(nobs*nsim), nrow = nobs)     


nsimul <- 10   # number random draws for the converging algorithm
#  preference shock, once drawn, fix across all the iterations 
eps1ct <-  runif(nobs)     #----- random draws for choice proportions
eps2ct <-  runif(nobs)     #----- random draws for preference difference

# simulate the two choice counterfactual

uboot <- ubootsamp
e1<-eps1ct
e2<-eps2ct

smallnumber <- 0.001
R <- 2
probSIM <- vecpref[,c(5,6)]
cfu1 <- PARgrade[[3]]
cfu1m2 <- PARgrade[[2]][9]
m1h <- mft[,1]
m2h <- mft[,2]
Enem <- uboot[[4]]

PSob <- matrix(rep(0),nrow=nobs, ncol=nsimul)
PFort <- matrix(rep(0),nrow=nobs, ncol=nsimul)
tsim <- matrix(rep(0), nrow=nsimul, ncol=4)
SobSIM <- matrix(rep(0),nrow=nobs, ncol=nsimul)
convg <- rep(0, nsimul)

# start the iteration 
for (i in 1:nsimul)
{ 
  Sobsim <-FUNchoice(e1,e2,vecpref, probSIM, uboot)
  
  u1 <- sample(uboot[[1]], size = nobs*nresim, replace=TRUE)
  u2 <- sample(uboot[[2]], size = nobs*nresim, replace=TRUE) 
  
  u1 <- matrix(u1, nrow=nobs, ncol=nresim)
  u2 <- matrix(u2, nrow =nobs, ncol=nresim)
  # simulate the success probabilities 
  ps<-foreach(j=(1:nresim), .combine="cbind") %dopar%
  {
    
    # the simulated grades
    m1s<-m1h+u1[,j]  
    # the combination for ranking
    m1sk<-m1s+120*Enem/63
    # the second stage grades 
    m2s<-m2h+cfu1m2*u1[,j] + u2[,j]  
    # the combination for ranking
    m2sk<-0.4*m1sk +0.6*m2s
    # divide the population by their choices in the sample
    # chances are that the people chose Sobral is small than the admission number
    t1sk <- sort(m1sk[Sobsim==1], decreasing=TRUE)
    t1s  <- max(t1sk[nos*R], min(t1sk),na.rm=TRUE)
    t1fk <- sort(m1sk[Sobsim==0], decreasing=TRUE) 
    t1f  <- max(t1fk[nof*R], min(t1sk),na.rm=TRUE)
    # second stage
    t2sk <- sort(m2sk[Sobsim==1 & m1sk>=t1s], decreasing=TRUE)
    t2s  <- max(t2sk[nos], min(t2sk), na.rm=TRUE) 
    t2fk <- sort(m2sk[Sobsim==0 & m1sk>=t1f], decreasing=TRUE)
    t2f  <- max(t2fk[nof], min(t2fk), na.rm=TRUE) 
    
    #-------- Success probabilities --------------------------------------------------------
    # for sobral
    pass_sob <- (m1sk >= t1s & m2sk >= t2s )
    # for fortaleza
    pass_fort<- (m1sk >= t1f & m2sk >= t2f )
    #get them together
    c(pass_sob, pass_fort,t1s, t1f, t2s, t2f)
  }
  
  #compute the success probability from the number of passing in all simulations. 
  avps <- apply(ps, 1, mean )
  
  PSob[,i]<-avps[1:nobs]
  PFort[,i]<-avps[(nobs+1):(nobs*2)]
  
  tsim[i,]<-avps[(nobs*2+1):length(avps)]
  
  convg[i]<-sum((PSob[,i] - probSIM[,1])^2 + (PFort[,i] - probSIM[,2])^2 > smallnumber )
  probSIM<-cbind(PSob[,i],PFort[,i]) 
  Sobsim2 <- FUNchoice(e1,e2,vecpref, probSIM, uboot)
  Choix <- sum(abs(Sobsim - Sobsim2))
  if (convg[i] <10.5)
    break
}
SIMsysARC<-list(convg, PSob, PFort,probSIM,Sobsim)

# stop the clustering clears the ram. remmeber to do it everytime
stopCluster(cl)

# save the simulation results -- which is only dependent on one random draw of epsilons

save(SIMsysARC, file = "CMX_CountResult_ARC.RData")

# load the data if you don't want to re-run it
load("CMX_CountResult_ARC.rdata")

# check convergence
CONVGARC <- SIMsysARC[[1]]
CONVGARC <- CONVGARC[CONVGARC!=0]

# the convergent success probabilities  
PSIMARC  <- SIMsysARC[[4]]

# the choice of the programs
dARC <- SIMsysARC[[5]]

# RE-simulate the counterfactual system to get the distributions of thresholds 
simulARC <- matrix(runif(nobs*nsimT), nrow = nobs)   #random draw from uniform distribution

# data for simulation of thresholds
datsimARC <- list(dARC, mft[,1], mft[,2])

# simulated thresholds 
TSIMARCRESIM <- FUNresimARC(datsimARC,nsimT,ubootsamp,simulARC,PARgrade)
TARC <- apply(TSIMARCRESIM, 2, mean)
TARCsd <- apply(TSIMARCRESIM, 2, sd)


#######################################################################################
############  utility comparison of admission rate change countefactual ###############
#######################################################################################
attach(prep_boot)


Sob <- (Number ==48) # the choice in the original system 
# coefficient mu in (0.5, 1)
mu <- 0.5
# utility difference in the sub-population where both programs offer positive utility
UTARCSF <- exp(logvdiff)
# utility in sob > 0 and fort < 0 
UTARCSob <- exp(logvdiff + 0.5)*mu

# number of simulations used for computing expected utility
cl <- makeCluster(3)  # set the number of cpu you want to use
registerDoParallel(cl)

nobut <- 999

UTARCPOST <- foreach(i = (1:nobut),  .combine=cbind)%dopar%
{
  #  preference shock, once drawn, fix across all the iterations 
  eps1ct <-  runif(nobs)     #----- random draws for choice proportions
  eps2ct <-  runif(nobs)     #----- random draws for preference difference
  # simulate the timing change counterfactual
  e1<-eps1ct
  e2<-eps2ct
  # choices in each iteration
  choix<-FUNchoice(e1,e2,vecpref, PSIMARC, uboot)
   
  # individual utility in the two choice counterfactual system 
  PSIMARC[,1]*(UTARCSob*delta10/(delta10+delta11) + UTARCSF*delta11/(delta10+delta11))*Sob  + PSIMARC[,1]*1*(1-Sob)   

} 
stopCluster(cl)
utilpostARC<- apply(UTARCPOST,1,mean)

# individual utility in the original system 
utilorg <- p2sob*(UTARCSob*delta10/(delta10+delta11) + UTARCSF*delta11/(delta10+delta11))*Sob  + p2fort*1*(1-Sob)   

# the utility difference
utildiffARC <- utilpostARC -utilorg

# the Final Grade -- the grade combination which is use to evalue the final admission
# this is taken as the ability proxy for the utility comparison exercise
FG <- 0.4*(m1p+120*Enem/63) + 0.6*m2p

#======= study the utility difference in difference percentage intervals =======================
#===============================================================================================

# get the percentiles of the final grades 
qtfg<-quantile(FG, probs=seq(0,1, 0.02))

# utility difference for all students
utildiffall<-rep(0,length(qtfg)-1)
utildiffallsd<-rep(0, length(qtfg)-1)
for (i in 1:length(qtfg)-1)
{
  utildiffall[i]<-mean( utildiffARC[(FG>=qtfg[i] & FG<qtfg[i+1])]) # mean utility difference between (i-1)*% and i%
  utildiffallsd[i]<-sd( utildiffARC[FG>=qtfg[i] & FG<qtfg[i+1]])   # stand deviation of utility difference between (i-1)*% and i%
}

# utility difference for those chose sobral 
utildiffsob<-rep(0,length(qtfg)-1)
utildiffsobsd<-rep(0, length(qtfg)-1)
for (i in 1:length(qtfg)-1)
{
  utildiffsob[i]<-mean( utildiffARC[(FG>=qtfg[i] & FG<qtfg[i+1] & Sob==1)])
  utildiffsobsd[i]<-sd( utildiffARC[FG>=qtfg[i] & FG<qtfg[i+1]  & Sob==1])
}

# utility difference for those chose fortaleza
utildifffort<-rep(0,length(qtfg)-1)
utildifffortsd<-rep(0, length(qtfg)-1)
for (i in 1:length(qtfg)-1)
{
  utildifffort[i]<-mean( utildiffARC[(FG>=qtfg[i] & FG<qtfg[i+1] & Sob==0)])
  utildifffortsd[i]<-sd( utildiffARC[FG>=qtfg[i] & FG<qtfg[i+1] & Sob==0])
}

#  utility difference in different intervals of interest 
D1<-c(mean( utildiffARC[FG>=qtfg[1] & FG<qtfg[25+1]]),
      sd( utildiffARC[FG>=qtfg[1] & FG<qtfg[25+1]]),
      mean( utildiffARC[FG >=qtfg[1] & FG<qtfg[25+1] & Sob==1]),
      sd( utildiffARC[FG >=qtfg[1] & FG<qtfg[25+1] & Sob==1]),
      mean( utildiffARC[FG >=qtfg[1] & FG<qtfg[25+1] & Sob==0]),
      sd( utildiffARC[FG >=qtfg[1] & FG<qtfg[25+1] & Sob==0])
)        # in the interval of below average -- in which the students have nearly 0 chance of being admitted

D2<-c(mean( utildiffARC[FG>=qtfg[26] & FG<qtfg[30+1]]),
      sd( utildiffARC[FG>=qtfg[26] & FG<qtfg[30+1]]),
      mean( utildiffARC[FG >=qtfg[26] & FG<qtfg[30+1] & Sob==1]),
      sd( utildiffARC[FG >=qtfg[26] & FG<qtfg[30+1] & Sob==1]),
      mean( utildiffARC[FG >=qtfg[26] & FG<qtfg[30+1] & Sob==0]),
      sd( utildiffARC[FG >=qtfg[26] & FG<qtfg[30+1] & Sob==0])
)       # from 50% to 60% 
D3<-c(mean( utildiffARC[FG>=qtfg[31] & FG<qtfg[35+1]]),
      sd( utildiffARC[FG>=qtfg[31] & FG<qtfg[35+1]]),
      mean( utildiffARC[FG >=qtfg[31] & FG<qtfg[35+1] & Sob==1]),
      sd( utildiffARC[FG >=qtfg[31] & FG<qtfg[35+1] & Sob==1]),
      mean( utildiffARC[FG >=qtfg[31] & FG<qtfg[35+1] & Sob==0]),
      sd( utildiffARC[FG >=qtfg[31] & FG<qtfg[35+1] & Sob==0])
)       # from 60% to 70%
D4<-c(mean( utildiffARC[FG>=qtfg[36] & FG<qtfg[40+1]]),
      sd( utildiffARC[FG>=qtfg[36] & FG<qtfg[40+1]]),
      mean( utildiffARC[FG >=qtfg[36] & FG<qtfg[40+1] & Sob==1]),
      sd( utildiffARC[FG >=qtfg[36] & FG<qtfg[40+1] & Sob==1]),
      mean( utildiffARC[FG >=qtfg[36] & FG<qtfg[40+1] & Sob==0]),
      sd( utildiffARC[FG >=qtfg[36] & FG<qtfg[40+1] & Sob==0])
)

Dcomp<-rbind(D1,D2,D3,D4, 
             cbind(utildiffall[41:50],utildiffallsd[41:50],
                   utildiffsob[41:50],utildiffsobsd[41:50],
                   utildifffort[41:50], utildifffortsd[41:50]))



##############################################################################################################
#========================= Output of Adminssion rate change: Table and Figure ================================
##############################################################################################################

#========================== Table 4 ==========================================================================
T1 <- c(mean(utildiffARC),NA,mean(utildiffARC[Sob==1]),NA,mean(utildiffARC[Sob==0]),NA)
T2 <- c(sd(utildiffARC),NA,sd(utildiffARC[Sob==1]),NA,sd(utildiffARC[Sob==0]),NA)
T3 <- c(sum(utildiffARC >0)/nobs,NA,sum(utildiffARC[Sob==1]>0)/sum(Sob==1),NA,sum(utildiffARC[Sob==0]>0)/sum(Sob==0),NA)

table_4 <- rbind(Dcomp, T1,T2,T3)

colnames(table_4) <- c("All mean","All s.d.","Sobral mean","Sobral s.d.","Fortaleza mean","Fortaleza s.d." )
rownames(table_4) <- c("0%-50%","50%-60%","60%-70%","70%-80%","80%-82%","82%-84%","84%-86%","86%-88%","88%-90%","90%-92%","92%-94%","94%-96%","96%-98%","98%-100%","Sample mean","Sample sd","Sample positive")

print(table_4, digits=2)


#========================= Figure 2 ==========================================================================
#probability change sob
pchangeSob <- PSIMARC[,1]-p2sob
changesob <- cbind(FG,pchangeSob)
pchangeSob1<- subset( changesob, Sob==1)
pchangeSob2<- subset( changesob, Sob==0)

#probability different plot
#pdf("spcARCsob.pdf")   optional pdf output code
plot(FG, pchangeSob, ylim=c(-0.05,0.075),pch=1, cex=0.4,cex.axis=0.8, cex.lab=0.8, col='deepskyblue3', ann=FALSE)
lines(lowess(FG, pchangeSob, f =0.15, iter=5), lwd=2, col='red')
abline(h=0, lty=4, lwd=2, col='gray29') 
abline(v=qtfg[26], lty=4, col='orange')
abline(v=quantile(FG, probs=(1-4*(nos+nof)/nobs)), lty=3, col='purple')
abline(v=quantile(FG, probs=(1-(nos+nof)/nobs)), lty =1, col='blue')
title(ylab="Success probability change")
title(xlab="Expected final grade")
#dev.off()

 
#========================= Figure S.iv ==========================================================================
#probability change fortaleza
pchangeFort <- PSIMARC[,2]-p2fort
changefort <- cbind(FG,pchangeFort)

# the plot
#pdf("spcARCfort_V1.pdf")   optional pdf output code
plot(FG, pchangeFort, ylim=c(-0.05,0.075),pch=1, cex=0.4, cex.axis=0.8, cex.lab=0.8, col='deepskyblue3', ann=FALSE)
lines(lowess(FG, pchangeFort, f =0.15, iter=5), lwd=2, col='red')
abline(v=qtfg[26], lty=4, col='orange')
abline(h=0, lty=4, lwd=2, col='gray29') 
abline(v=quantile(FG, probs=(1-4*(nos+nof)/nobs)), lty=3, col='purple')
abline(v=quantile(FG, probs=(1-(nos+nof)/nobs)), lty =1, col='blue')
title(ylab="Success probability change")
title(xlab="Expected final grade")
#dev.off()


#======================== Figure S.v  ==========================================================================
mARC<-cbind(FG, utildiffARC)
mARCs<-subset(mARC, Sob==1)
mARCf<-subset(mARC, Sob==0)

#pdf("utcARC.pdf")   optional pdf output code
plot(mARCf[,1],mARCf[,2], pch=2, ylim=c(-0.04, 0.2),cex=0.4, col="deepskyblue3",ann=FALSE)
lines(lowess(mARCf[,1],mARCf[,2], f =0.15, iter=5), lwd=2, col='darkblue')
points(mARCs[,1],mARCs[,2], pch=5, cex=0.4, col="mistyrose3",ann=FALSE)
lines(lowess(mARCs[,1],mARCs[,2], f =0.15, iter=5), lty =6, lwd=2, col='salmon3')
abline(h=0, col='gray29')
abline(v=qtfg[26], lty=4, col='orange')
abline(v=quantile(FG, probs=(1-4*(nos+nof)/nobs)), lty=3, col='purple')
abline(v=quantile(FG, probs=(1-(nos+nof)/nobs)), lty =1, col='royalblue4')
title(xlab="Expected final grade")
title(ylab="Utility difference")
#dev.off()
