###################################################################################################################
##  QE_MS951   "College Choice, Selection and Allocation Mechanisms: A Structural Empirical Analysis"       ####### 
##  Replication files: all code is run in R, the main data file is .dta format                                 ####
###################################################################################################################

#---------------------- load the R packages necessary for the following exercises --------------------------------
library(nlme) 
library(foreign)
library(stats)
library(nloptr)
library(doParallel)
library(splines)
# for the paralel computing
cl <- makeCluster(24)  # set the number of cpu you want to use
registerDoParallel(cl)


#---------------------  Path and source files --------------------------------------------------------------------
setwd("G:/QXG/R/CMX2018QE")
# sources of subroutines
source("CMX_Proc_Data.R")
source("CMX_Proc_Resim.R")
source("CMX_Proc_Sucprob.R")
source("CMX_Proc_Lik.R")
source("CMX_Proc_Choice.R")


#==================================== Prepare the data ============================================================
# import the data from the base_2004.dta data file.
don <- read.dta("base_2004.dta", convert.factors =TRUE)
# we only keep the medical school students with non-missing first stage national genral knowledge test grades 
dat <- subset(don, (is.na(FS_FSE)==FALSE & Number<=49 & Number>=48))
attach(dat)
colnames(dat)  # check the variables available

#----------------- Basic data house keeping with the explanatory variables   --------------------------------------
# the minimum age of entering the exam is 16, we treat all missing age or reported 0 as the youngest, limited obs
age[is.na(age)]  <- 16
age[age==0]      <- 16
# clean the middle school type
em_priv[is.na( em_priv)] <- 0 
ef_priv[is.na( ef_priv)] <- 0 
em_supl[is.na( em_supl)] <- 0 
prep[is.na( prep)]       <- 0 
repet <- rep
repet[is.na( rep)]   <- 0 
educf[is.na( educf)] <- 0 
educm[is.na( educm)] <- 0 
# location and gender dummies
CFF    <- (1-interior-fora)
CFFfem <- CFF*female
CFFedm <- CFF*educm
femint <- female*interior
edint  <- educm*interior

#--------------------- Imputation Enem missing ------------------------------------------------------------------------
epsdat <- imp(dat)[[1]]      # normalized Enem scores are taken as the proxy of ability here 
# NaN warnings are due to 0 in logistic transformation, which we replace in the subroutine
nobs   <- length(epsdat)     # record the number of observations

# location and ability interaction term 
CFFeps <- CFF*epsdat
# Splines of ability residuals
bseps <- bs(epsdat, df = 2, degree = 2)

#------- The grades used to determine admission ----------------------------------------------------------------------- 
m1 <- FS_FSE - 120*Enem/63
m2 <- FS_SSE


#=================== First grade equation OLS: Table S.iv   ===========================================================
# the first specification
lm1sp1 <- lm(m1 ~ female+age+em_supl+em_priv+prep+repet+CFF+bseps+CFFeps+educm)

# the second specification 
lm1sp2 <- lm(m1 ~ female+age+em_supl+em_priv+prep+repet+CFF+bseps+CFFeps+educm+educf)

# The third specification
lm1sp3 <- lm(m1 ~ female+age+em_supl+em_priv+prep+repet+CFF+CFFeps+educm+educf)

# outputs of the first grade equation
m1p    <- lm1sp1$fitted.values  # fitte value of m1
beta1  <- coef(lm1sp1)          # the estimated parameters
u1     <- lm1sp1$residuals      # the empirical residulas


#----------- specifications tests: optional   --------------------------------------------------------------------------
# test whether to keep both father and mother's education in the first grade equation 
# 1-pf(sum(lm1sp1$residuals^2-lm1sp2$residuals^2)/(2*sum(lm1sp2$residuals^2)/lm1sp2$df.residual), 2, lm1sp2$df.residual)

# the insignificant gender variable -- female, keep or not? 
#  F test the inclusion of female variable 
# lf<- lm(m1 ~ age+em_supl+em_priv+prep+repet+CFF+bseps+CFFeps+educm+educf)
# 1-pf(sum(lf$residuals^2-lm1sp2$residuals^2)/(2*sum(lm1sp2$residuals^2)/lm1sp2$df.residual), 1, lm1sp2$df.residual) 

#----------- Analysis of residuals: test the neccessity of heteroskedasticity ------------------  
# u1s <- (lm1sp1$residuals^2)
# summary(lm(u1s~ female+age+em_supl+em_priv+prep+repet+bseps+CFFeps+educm+educf))

#================= Second grade equation OLS: Table S.v ================================================================
# Splines of the residual of m1  
bsu1   <- bs(u1[!is.na(m2)], df = 2, degree = 2)
bsu1g  <- matrix(m2, nrow = nobs, ncol=dim(bsu1)[2]); bsu1g[!is.na(m2),] <- bsu1

# the first specification
lm2sp1 <- lm(m2 ~ female+age+em_supl+em_priv+prep+repet+epsdat+u1)     # missing m2 is automatically omitted

# the second specification 
lm2sp2 <- lm(m2 ~ female+age+em_supl+em_priv+prep+repet+epsdat+bsu1g)  # missing m2 is automatically omitted

# the outputs of second grade equation
beta2<-coef(lm2sp1)       # the estimated beta
m2p <- as.vector(cbind(rep(1,nobs), female, age, em_supl, em_priv, prep, repet, epsdat) %*% 
                   lm2sp1$coefficients[-length(lm2sp1$coefficients)])  # the fitte value of m2 
u2<-lm2sp1$residuals      # the empirical residuals

#------------ Analysis of residuals test the neccessity of heteroskedasticity ------------------
# u2s <- m2; u2s[!is.na(m2)] <- (lm2sp1$residuals^2)
# summary(lm(u2s~ female+age+em_supl+em_priv+prep+repet+epsdat+u1))


################################################################################################
############   ALL THE ESTIMATED RESULTS AND VARIABLES FOR THE FOLLOWING PROCCESS ##############
################################################################################################
# Distibution of residuals: Ordering  
u1o  <- sort(lm1sp1$residuals)              
u2o  <- sort(lm2sp1$residuals) 
# Control function parameter in the ranking combination 
cfu1 <- lm2sp1$coefficients[length(lm2sp1$coefficients)]+2/3   
# the fitted values for counterfactual study
mft  <- cbind(m1p,m2p)

#--------------------Prepare data for bootstrap  -----------------------------------------------------------------------
# store the explanatory variables
datagrade1<-data.frame(cbind(female,age,em_supl,em_priv,prep,repet,CFF,bseps,CFFeps,educm))
colnames(datagrade1)[8:9]<-c("bseps1","bseps2")
datagrade2<-data.frame(cbind(female,age,em_supl,em_priv,prep,repet,epsdat))
# store the residuals, choices and ability proxy for resampling
ubootsamp<-list(u1,u2,(Number==48),Enem)   
# store the parameters of grade estimation
PARgrade<-list(beta1, beta2, cfu1)
# store the data needed for the preference bootstrap
datapref<-cbind(epsdat,female,age,repet,educf,educm)


###########################################################################################################################
#     the success probability are simulated using the emprical distribution of errors of the thresholds. S.2.2.3          #
###########################################################################################################################
#   set the simulation parameters
set.seed(1894285437, kind = NULL)
nsim   <- 999                                      # number of simulations in computing the success probability: n
simul  <- matrix(runif(nobs*nsim), nrow = nobs)    # random draw from uniform distribution

nsimT  <- 2000                                     # the number of draws into the emprical error distributions: n_s 
simulT <- matrix(runif(nobs*nsimT), nrow = nobs)   # random draw from uniform distribution

# final number of seats
nos    <- 40
nof    <- 150
# number of seats at the second exams
nobsm2 <- sum(!is.na(m2))
# data for simulation of thresholds
datsimT<-cbind((Number==48), m1p, m2p)

# simulated thresholds for the success probabilites 
REsim <- FUNresim(datsimT,nsimT,ubootsamp,simul,PARgrade)

# the simulated thresholds and success probabilities. 
Tresim <- apply(REsim[[1]],1,mean)
p2sob  <- apply(REsim[[3]],1,mean)
p2fort <- apply(REsim[[5]],1,mean)

save(REsim, file="REsim.rdata")
# load("REsim.rdata")  # in case of not wanting to re-esimate everything

# the log success probability difference
w      <- log(apply(REsim[[2]], 1, mean)/apply(REsim[[3]], 1, mean)) 


#===================== prepare the data for preference estimation ==========================================================
dpref <- cbind((Number==48),w,epsdat,female,age,repet,educf,educm)
dta   <- as.matrix(dpref[, c(1:2,3,3:8)])

# -------   Number of Variables  ----------------------------------------
ndt     <- 2       # Number of variables in delta and delta1 
na      <- 7       # Number of variables in alpha   
lam     <- 5.e-4   # Penalization parameter
#--------  Starting values for Parameters  ------------------------------
cdt1  <- c(log(1/2), 0.01)
cdt   <- c(.05, 0.01)
ca    <- c(.05, 0.05, .005, 0.005, .005, .005, 0.01 )
coef0 <- c(cdt1, cdt, ca)

#-------  Check of gradient  ----------------------------------------------
check.derivatives(coef0, slikTP, gradlikTP, check_derivatives_tol = 1e-04, check_derivatives_print = 'all')

#   Estimation of preference  
valbl <- nloptr(coef0, eval_f = slikTP, eval_grad_f = gradlikTP,
                opts = list("algorithm" = "NLOPT_LD_LBFGS", "print_level" = 2,  "xtol_rel" = 1.0e-4,
                            "maxtime"=25000, "maxeval" = 2000))
bpref <- valbl$solution

# comupting the standard errors 
hess <- fdHess(bpref, slikTP)[[3]]
vcov <- solve(hess)
sdpref <- sqrt(diag(vcov))
#  p-values  
pval <- pnorm(-abs(bpref/sdpref))*2

#  getting the deltas  
delta10 <- 1/(1+exp(-cbind(rep(1,nobs), epsdat) %*% bpref[1:ndt]))
delta11 <- (1-delta10)/(1+exp(-cbind(rep(1,nobs), epsdat) %*% bpref[(ndt+1):(2*ndt)]))
delta01 <- 1-delta10 -delta11
#  the matrix of Deltas for each individual
DT <- cbind(delta10, delta11, delta01)

########################################################################################
#================= parameters for simulations and bootstrap   ==========================
########################################################################################
#  the parameters
Pardelta <- bpref[1:(2*ndt)]
gamma <- bpref[5:(ndt*2+na)]

#  set the boundaries 
uppref <- bpref+3*sdpref
lowpref <- bpref-3*sdpref
#  the log utility difference  
logvdiff <- as.matrix(cbind(rep(1,nobs), dpref[ ,(ndt+1):(na+ndt-1)]))%*%gamma

#  original success probability
ProbORG <- cbind(p2sob,p2fort)

#  the vectors needed for boostrap 
vecpref<-cbind(logvdiff,DT,ProbORG)

#  parameters for boostrap
PARpref<-list(bpref,uppref,lowpref)


# save all the parameters and vectors from all CMX_data, CMX_grades and CMX_preferece
prep_boot<-list(PARgrade, ubootsamp, datagrade1, datagrade2, PARpref, vecpref, datapref, mft)
save(prep_boot, file="~/R/CMX2018QE/Bootstrap/CMX_Prep_boot.rdata")

# stop the clustering clears the ram. remmeber to do it everytime
stopCluster(cl)




library(psych)

#====================================== Output Table 1 ==================================================================================
dat[(dat[,"Enem"] == 0), "Enem"]<- NA

table1up <- dat[Number==48, c("Enem","FS_FSE","FS_SSE","female","age","em_priv","prep","rep","educf","educm")]
table1low <- dat[Number==49, c("Enem","FS_FSE","FS_SSE","female","age","em_priv","prep","rep","educf","educm")]

describe(table1up, skew=FALSE, ranges = FALSE, quant = c(0, 0.5, 1),na.rm=TRUE)[, c(3,7,6,8,4,2)] 
describe(table1low, skew=FALSE, ranges = FALSE, quant = c(0, 0.5, 1),na.rm=TRUE)[, c(3,7,6,8,4,2)] 

table_1 <- rbind(describe(table1up, skew=FALSE, ranges = FALSE, quant = c(0, 0.5, 1),na.rm=TRUE)[, c(3,7,6,8,4,2)], 
                describe(table1low, skew=FALSE, ranges = FALSE, quant = c(0, 0.5, 1),na.rm=TRUE)[, c(3,7,6,8,4,2)] 
)
 
colnames(table_1)<- c("mean","median","min","max","s.d.","n")
print(table_1, digits=1)


#========================================= Output Table 2 ===============================================================================

p1s_table <- apply(REsim[[2]],1,mean)
p1f_table <- apply(REsim[[4]],1,mean)

p2s_table <- apply(REsim[[3]],1,mean)
p2f_table <- apply(REsim[[5]],1,mean)

#------------------------------- table 2--------------------------------------------------------------------------------------------------
table_2 <- data.frame(cbind( c(mean(as.vector(p1s_table)),quantile(p1s_table, probs=c(0, 0.25,  0.5, 0.75,1))),
                             c(mean(as.vector(p1f_table)),quantile(p1f_table, probs=c(0, 0.25,  0.5, 0.75,1))),
                             c(mean(as.vector(p2s_table)),quantile(p2s_table, probs=c(0, 0.25,  0.5, 0.75,1))),
                             c(mean(as.vector(p2f_table)),quantile(p2f_table, probs=c(0, 0.25,  0.5, 0.75,1)))))
colnames(table_2)<- c("Sobral 1st", "Sobral final", "Fortaleza 1st", "Fortaleza final")
rownames(table_2)[1]<- c("mean")
print(table_2, digits=3)












