###################################################################################################################
##  QE_MS951   "College Choice, Selection and Allocation Mechanisms: A Structural Empirical Analysis"       ####### 
##  Replication files: all code is run in R, the main data file is .dta format                                 ####
###################################################################################################################

#---------------------- load the R packages necessary for the following exercises --------------------------------
library(nlme) 
library(foreign)
library(stats)
library(nloptr)
library(doParallel)
library(splines)
# for the paralel computing
#cl <- makeCluster(4)  # set the number of cpu you want to use
#registerDoParallel(cl)


#---------------------  Path and source files --------------------------------------------------------------------
setwd("~/R/CMX2018QE")
# sources of subroutines
source("CMX_Proc_Data.R")
source("CMX_Proc_Resim.R")
source("CMX_Proc_Sucprob.R")
source("CMX_Proc_Lik.R")
source("CMX_Proc_Choice.R")


#==================================== Prepare the data ============================================================
# import the data from the base_2004.dta data file.
don <- read.dta("base_2004.dta", convert.factors =TRUE)
# we only keep the medical school students with non-missing first stage national genral knowledge test grades 
dat <- subset(don, (is.na(FS_FSE)==FALSE & Number<=49 & Number>=48))
attach(dat)
colnames(dat)  # check the variables available

#----------------- Basic data house keeping with the explanatory variables   --------------------------------------
# the minimum age of entering the exam is 16, we treat all missing age or reported 0 as the youngest, limited obs
age[is.na(age)]  <- 16
age[age==0]      <- 16
# clean the middle school type
em_priv[is.na( em_priv)] <- 0 
ef_priv[is.na( ef_priv)] <- 0 
em_supl[is.na( em_supl)] <- 0 
prep[is.na( prep)]       <- 0 
repet <- rep
repet[is.na( rep)]   <- 0 
educf[is.na( educf)] <- 0 
educm[is.na( educm)] <- 0 
# location and gender dummies
CFF    <- (1-interior-fora)
CFFfem <- CFF*female
CFFedm <- CFF*educm
femint <- female*interior
edint  <- educm*interior

#--------------------- Imputation Enem missing ------------------------------------------------------------------------
epsdat <- imp(dat)[[1]]      # normalized Enem scores are taken as the proxy of ability here 
# NaN warnings are due to 0 in logistic transformation, which we replace in the subroutine
nobs   <- length(epsdat)     # record the number of observations

# location and ability interaction term 
CFFeps <- CFF*epsdat
# Splines of ability residuals
bseps <- bs(epsdat, df = 2, degree = 2)

#------- The grades used to determine admission ----------------------------------------------------------------------- 
m1 <- FS_FSE - 120*Enem/63
m2 <- FS_SSE


#=================== First grade equation OLS: Table S.iv   ===========================================================
# the first specification
lm1sp1 <- lm(m1 ~ female+age+em_supl+em_priv+prep+repet+bseps+CFF+CFFeps)

# the second specification 
lm1sp2 <- lm(m1 ~ female+age+em_supl+em_priv+prep+repet+bseps+CFF+CFFeps+educm+educf)

# The third specification
lm1sp3 <- lm(m1 ~ female+age+em_supl+em_priv+prep+repet+epsdat+CFF+CFFeps+educm+educf)

# outputs of the first grade equation
m1p    <- lm1sp1$fitted.values  # fitte value of m1
beta1  <- coef(lm1sp1)          # the estimated parameters
u1     <- lm1sp1$residuals      # the empirical residulas


#----------- specifications tests: optional   --------------------------------------------------------------------------
# test whether to keep both father and mother's education in the first grade equation 
# 1-pf(sum(lm1sp1$residuals^2-lm1sp2$residuals^2)/(2*sum(lm1sp2$residuals^2)/lm1sp2$df.residual), 2, lm1sp2$df.residual)

# the insignificant gender variable -- female, keep or not? 
#  F test the inclusion of female variable 
# lf<- lm(m1 ~ age+em_supl+em_priv+prep+repet+CFF+bseps+CFFeps+educm+educf)
# 1-pf(sum(lf$residuals^2-lm1sp2$residuals^2)/(2*sum(lm1sp2$residuals^2)/lm1sp2$df.residual), 1, lm1sp2$df.residual) 

#----------- Analysis of residuals: test the neccessity of heteroskedasticity ------------------  
# u1s <- (lm1sp1$residuals^2)
# summary(lm(u1s~ female+age+em_supl+em_priv+prep+repet+bseps+CFFeps+educm+educf))

#================= Second grade equation OLS: Table S.v ================================================================
# Splines of the residual of m1  
bsu1   <- bs(u1[!is.na(m2)], df = 2, degree = 2)
bsu1g  <- matrix(m2, nrow = nobs, ncol=dim(bsu1)[2]); bsu1g[!is.na(m2),] <- bsu1

# the first specification
lm2sp1 <- lm(m2 ~ female+age+em_supl+em_priv+prep+repet+u1+epsdat)     # missing m2 is automatically omitted

# the second specification 
lm2sp2 <- lm(m2 ~ female+age+em_supl+em_priv+prep+repet+bsu1g+epsdat)  # missing m2 is automatically omitted

# the outputs of second grade equation
beta2<-coef(lm2sp1)       # the estimated beta
m2p <- as.vector(cbind(rep(1,nobs), female, age, em_supl, em_priv, prep, repet, epsdat) %*% 
                   lm2sp1$coefficients[-length(lm2sp1$coefficients)])  # the fitte value of m2 
u2<-lm2sp1$residuals      # the empirical residuals

#------------ Analysis of residuals test the neccessity of heteroskedasticity ------------------
# u2s <- m2; u2s[!is.na(m2)] <- (lm2sp1$residuals^2)
# summary(lm(u2s~ female+age+em_supl+em_priv+prep+repet+epsdat+u1))


################################################################################################
############   ALL THE ESTIMATED RESULTS AND VARIABLES FOR THE FOLLOWING PROCCESS ##############
################################################################################################
# Distibution of residuals: Ordering  
u1o  <- sort(lm1sp1$residuals)              
u2o  <- sort(lm2sp1$residuals) 
# Control function parameter in the ranking combination 
cfu1 <- lm2sp1$coefficients[length(lm2sp1$coefficients)]+2/3   
# the fitted values for counterfactual study
mft  <- cbind(m1p,m2p)

#--------------------Prepare data for bootstrap  -----------------------------------------------------------------------
# store the explanatory variables
datagrade1<-data.frame(cbind(female,age,em_supl,em_priv,prep,repet,CFF,bseps,CFFeps,educm))
colnames(datagrade1)[8:9]<-c("bseps1","bseps2")
datagrade2<-data.frame(cbind(female,age,em_supl,em_priv,prep,repet,epsdat))
# store the residuals, choices and ability proxy for resampling
ubootsamp<-list(u1,u2,(Number==48),Enem)   
# store the parameters of grade estimation
PARgrade<-list(beta1, beta2, cfu1)
# store the data needed for the preference bootstrap
datapref<-cbind(epsdat,female,age,repet,educf,educm)


###########################################################################################################################
#     the success probability are simulated using the emprical distribution of errors of the thresholds. S.2.2.3          #
###########################################################################################################################
#   set the simulation parameters
set.seed(1894285437, kind = NULL)
nsim   <- 999                                      # number of simulations in computing the success probability: n
simul  <- matrix(runif(nobs*nsim), nrow = nobs)    # random draw from uniform distribution

nsimT  <- 2000                                     # the number of draws into the emprical error distributions: n_s 
simulT <- matrix(runif(nobs*nsimT), nrow = nobs)   # random draw from uniform distribution

# final number of seats
nos    <- 40
nof    <- 150
# number of seats at the second exams
nobsm2 <- sum(!is.na(m2))
# data for simulation of thresholds
datsimT<-cbind((Number==48), m1p, m2p)

# simulated thresholds for the success probabilites 
#REsim <- FUNresim(datsimT,nsimT,ubootsamp,simul,PARgrade)
load("REsim.rdata") 
# the simulated thresholds and success probabilities. 
Tresim <- apply(REsim[[1]],1,mean)
p2sob  <- apply(REsim[[3]],1,mean)
p2fort <- apply(REsim[[5]],1,mean)

# save(REsim, file="REsim.rdata")
# load("REsim.rdata")  # in case of not wanting to re-esimate everything

# the log success probability difference
w      <- log(apply(REsim[[2]], 1, mean)/apply(REsim[[3]], 1, mean)) 


#===================== prepare the data for preference estimation ==========================================================
dpref <- cbind((Number==48),w,epsdat,female,age,repet,educf,educm)
dta   <- as.matrix(dpref[, c(1:2,3,3:8)])

# -------   Number of Variables  ----------------------------------------
ndt     <- 2       # Number of variables in delta and delta1 
na      <- 7       # Number of variables in alpha   
lam     <- 5.e-4   # Penalization parameter
#--------  Starting values for Parameters  ------------------------------
cdt1  <- c(log(1/2), 0.01)
cdt   <- c(.05, 0.01)
ca    <- c(.05, 0.05, .005, 0.005, .005, .005, 0.01 )
coef0 <- c(cdt1, cdt, ca)

#-------  Check of gradient  ----------------------------------------------
check.derivatives(coef0, slikTP, gradlikTP, check_derivatives_tol = 1e-04, check_derivatives_print = 'all')

#   Estimation of preference  
valbl <- nloptr(coef0, eval_f = slikTP, eval_grad_f = gradlikTP,
                opts = list("algorithm" = "NLOPT_LD_LBFGS", "print_level" = 2,  "xtol_rel" = 1.0e-4,
                            "maxtime"=25000, "maxeval" = 2000))
bpref <- valbl$solution

# comupting the standard errors 
hess <- fdHess(bpref, slikTP)[[3]]
vcov <- solve(hess)
sdpref <- sqrt(diag(vcov))
#  p-values  
pval <- pnorm(-abs(bpref/sdpref))*2

#  getting the deltas  
delta10 <- 1/(1+exp(-cbind(rep(1,nobs), epsdat) %*% bpref[1:ndt]))
delta11 <- (1-delta10)/(1+exp(-cbind(rep(1,nobs), epsdat) %*% bpref[(ndt+1):(2*ndt)]))
delta01 <- 1-delta10 -delta11
#  the matrix of Deltas for each individual
DT <- cbind(delta10, delta11, delta01)

########################################################################################
#================= parameters for simulations and bootstrap   ==========================
########################################################################################
#  the parameters
Pardelta <- bpref[1:(2*ndt)]
gamma <- bpref[5:(ndt*2+na)]

#  set the boundaries 
uppref <- bpref+3*sdpref
lowpref <- bpref-3*sdpref
#  the log utility difference  
logvdiff <- as.matrix(cbind(rep(1,nobs), dpref[ ,(ndt+1):(na+ndt-1)]))%*%gamma

#  original success probability
ProbORG <- cbind(p2sob,p2fort)

#  the vectors needed for boostrap 
vecpref<-cbind(logvdiff,DT,ProbORG)

#  parameters for boostrap
PARpref<-list(bpref,uppref,lowpref)

# save all the parameters and vectors from all CMX_data, CMX_grades and CMX_preferece
prep_boot<-list(PARgrade, ubootsamp, datagrade1, datagrade2, PARpref, vecpref, datapref, mft)

# stop the clustering clears the ram. remmeber to do it everytime
#stopCluster(cl)



##################################################################################################################################
#           Table and figure output for data appendix S.                                                                         #
##################################################################################################################################
detach(package:plyr)
library(dplyr)
library(stargazer)

#======================================= Table S.i. =============================================================================
print(
table.S.i <- don %>% group_by(Subgroup) %>% summarise(
    Applications=length(Number),
    Pass_1st_stage=sum((SIT1=="A              "))/length(Number),
    Pass_2nd_stage=sum((SIT2=="A              "))/length(Number),
    Positions=sum((SIT2=="A              "))
  )  
%>% as.data.frame()
,     digits=4
)

#===================================== Tables S.ii ==================================================================================
print(
table.S.ii <- don %>% group_by(Subgroup) %>%
  summarise(
    tenth_pt_all=quantile(FS_FSE, probs = .1, na.rm = TRUE, names = TRUE),
    min_1st_stage=min(FS_FSE[(SIT1=="A              ")], na.rm = TRUE),
    min_pass=min(FS_FSE[(SIT2=="A              ")], na.rm = TRUE),
    median_all=median(FS_FSE,  na.rm = TRUE ),
    median_1st_stage=median(FS_FSE[(SIT1=="A              ")], na.rm = TRUE),
    median_pass=median(FS_FSE[(SIT2=="A              ")], na.rm = TRUE),
    max_all=max(FS_FSE,  na.rm = TRUE ),
    max_1st_stage=max(FS_FSE[(SIT1=="A              ")], na.rm = TRUE),
    max_pass=max(FS_FSE[(SIT2=="A              ")], na.rm = TRUE)
    )%>% as.data.frame()
   ,     digits=4
) 

#================================= Table S.iii: 48 is Sobral and 49 is Fortaleza ======================================================
print(
table.S.iii <-  dat %>% group_by(Number) %>%
    summarise(
      tenth_pt_all=quantile(FS_FSE, probs = .1, na.rm = TRUE, names = TRUE),
      min_1st_stage=min(FS_FSE[(SIT1=="A              ")], na.rm = TRUE),
      min_pass=min(FS_FSE[(SIT2=="A              ")], na.rm = TRUE),
      median_all=median(FS_FSE,  na.rm = TRUE ),
      median_1st_stage=median(FS_FSE[(SIT1=="A              ")], na.rm = TRUE),
      median_pass=median(FS_FSE[(SIT2=="A              ")], na.rm = TRUE),
      max_all=max(FS_FSE,  na.rm = TRUE ),
      max_1st_stage=max(FS_FSE[(SIT1=="A              ")], na.rm = TRUE),
      max_pass=max(FS_FSE[(SIT2=="A              ")], na.rm = TRUE),
      observations=n()
    )%>% as.data.frame()
  ,     digits=5
) 


#=========================================== Table S.iv ==============================================================================
table.S.iv <- stargazer(lm1sp1,lm1sp2,lm1sp3, type = "text", title="First Stage Exam grade equation",column.labels=c("specification 1", "specification 2", "specification 3"), align=TRUE)

#============================================= Table S.v ==============================================================================
table.S.v <- stargazer(lm2sp1,lm2sp2, type = "text", title="Second Stage Exam grade equation",column.labels=c("specification 1", "specification 2" ), align=TRUE)

#===================================== Table S.vi  and Table 2=========================================================================
#REsim_table <- FUNresim(datsimT,nsimT,ubootsamp,simul,PARgrade)
#save(REsim_table.rdata)
#load(REsim_table.rdata)
REsim_table <- REsim

p1s_table <- apply(REsim_table[[2]],1,mean)
p1f_table <- apply(REsim_table[[4]],1,mean)

p2s_table <- apply(REsim_table[[3]],1,mean)
p2f_table <- apply(REsim_table[[5]],1,mean)

#------------------------------- table 2--------------------------------------------------------------------------------------------------
table_2 <- data.frame(cbind( c(mean(as.vector(p1s_table)),quantile(p1s_table, probs=c(0, 0.25,  0.5, 0.75,1))),
       c(mean(as.vector(p1f_table)),quantile(p1f_table, probs=c(0, 0.25,  0.5, 0.75,1))),
       c(mean(as.vector(p2s_table)),quantile(p2s_table, probs=c(0, 0.25,  0.5, 0.75,1))),
       c(mean(as.vector(p2f_table)),quantile(p2f_table, probs=c(0, 0.25,  0.5, 0.75,1)))))
colnames(table_2)<- c("Sobral 1st", "Sobral final", "Fortaleza 1st", "Fortaleza final")
rownames(table_2)[1]<- c("mean")
print(table_2, digits=3)



#=================================== table S.vi ===================================================================================
# get the percentiles of the final grades 
qtfs_sob  <-quantile(FS_FSE[Number==48], probs=c(0,0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.75,0.8,0.85,0.9,0.95,1))
qtfs_fort <-quantile(FS_FSE[Number==49], probs=c(0,0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.75,0.8,0.85,0.9,0.95,1))

# utility difference for all students
odd1<-rep(0,length(qtfs_sob)-1)
odd2<-rep(0, length(qtfs_sob)-1)
for (i in 1:length(qtfs_sob)-1)
{
  odd1[i]<-mean((p1s_table[Number==48&FS_FSE>=qtfs_sob[i] & FS_FSE<qtfs_sob[i+1]]/p1f_table[Number==48&FS_FSE>=qtfs_sob[i] & FS_FSE<qtfs_sob[i+1]]))/mean((p1s_table[Number==49&FS_FSE>=qtfs_fort[i] & FS_FSE<qtfs_fort[i+1]]/p1f_table[Number==49&FS_FSE>=qtfs_fort[i] & FS_FSE<qtfs_fort[i+1]]))
  odd2[i]<-mean((p2s_table[Number==48&FS_FSE>=qtfs_sob[i] & FS_FSE<qtfs_sob[i+1]]/p2f_table[Number==48&FS_FSE>=qtfs_sob[i] & FS_FSE<qtfs_sob[i+1]]))/mean((p2s_table[Number==49&FS_FSE>=qtfs_fort[i] & FS_FSE<qtfs_fort[i+1]]/p2f_table[Number==49&FS_FSE>=qtfs_fort[i] & FS_FSE<qtfs_fort[i+1]]))
  }

table_S_vi <- cbind(c(0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.75,0.8,0.85,0.9,0.95,1), odd1,odd2)
print(table_S_vi, digits=3)




#====================================Table S.vii================================================================================

########################################################################################
##### specification 2 of preference 
# prepare the data
dprefsp2 <- cbind((Number==48),w,epsdat,CFF,female,age,repet,educf,educm)
dta <- as.matrix(dprefsp2[, c(1:2,3:4,3:9)])

#      Number of Variables 
ndt <- 3       # Number of variables in delta and delta1 
na <- 8       # Number of variables in alpha   
lam <- 5.e-4   # Penalization parameter
#    Starting values for Parameters  
cdt1 <- c(log(1/2), 0.01, -1)
cdt <- c(.05, 0.01, -1)
ca <- c(-0.5,0.05, 0.05, .005, 0.005, .005, .005, 0.1 )
coef0 <- c(cdt1, cdt, ca)

#   Check of gradient  
check.derivatives(coef0, slikTP, gradlikTP, check_derivatives_tol = 1e-04, check_derivatives_print = 'all')

#   Estimation of preference  
valbl <- nloptr(coef0, eval_f = slikTP, eval_grad_f = gradlikTP,
                opts = list("algorithm" = "NLOPT_LD_LBFGS", "print_level" = 0,  "xtol_rel" = 1.0e-4,
                            "maxtime"=25000, "maxeval" = 2000))
bprefsp2 <- valbl$solution

# comupting the standard errors 
hesssp2 <- fdHess(bprefsp2, slikTP)[[3]]
vcovsp2 <- solve(hesssp2)
sdprefsp2 <- sqrt(diag(vcovsp2))
#  p values  
pvalsp2 <- pnorm(-abs(bprefsp2/sdprefsp2))*2

#  getting the deltas  
delta10sp2 <- 1/(1+exp(-cbind(rep(1,nobs), epsdat, CFF) %*% bprefsp2[1:ndt]))
delta11sp2 <- (1-delta10sp2)/(1+exp(-cbind(rep(1,nobs), epsdat, CFF) %*% bprefsp2[(ndt+1):(2*ndt)]))
delta01sp2 <- 1-delta10sp2 -delta11sp2
#  the matrix of Deltas for each individual
DTsp2 <- cbind(delta10sp2, delta11sp2, delta01sp2)

########################################################################################
#================= parameters for simulations and bootstrap   ==========================
########################################################################################
#  the parameters
Pardeltasp2 <- bprefsp2[1:(2*ndt)]
gammasp2 <- bprefsp2[7:(ndt*2+na)]

#  set the boundaries 
upprefsp2 <- bprefsp2+3*sdprefsp2
lowprefsp2 <- bprefsp2-3*sdprefsp2
#  the log utility difference  
logvdiffsp2 <- as.matrix(cbind(rep(1,nobs), dprefsp2[ ,3:9]))%*%gammasp2

#  original success probability
ProbORG <- cbind(p2sob,p2fort)

#  the vectors needed for boostrap 
vecprefsp2<-cbind(logvdiffsp2,DTsp2,ProbORG)

#  parameters for boostrap
PARprefsp2<-list(bprefsp2,upprefsp2,lowprefsp2)
dataprefsp2<-cbind(epsdat,CFF,female,age,repet,educf,educm) 


########################################################################################
############ specification 2 of preference  ############################################
########################################################################################
# prepare the data
dprefsp3 <- cbind((Number==48),w,epsdat,CFF,female,age,repet,educf,educm)
dta <- as.matrix(dprefsp3[, c(1:2,4,3:9)])

#      Number of Variables 
ndt <- 2       # Number of variables in delta and delta1 
na <- 8       # Number of variables in alpha   
lam <- 5.e-4   # Penalization parameter
#    Starting values for Parameters  
cdt1 <- c(log(1/2), -1)
cdt <- c(.05,  -1)
ca <- c(-0.5,0.05, 0.05, .005, 0.005, .005, .005, 0.1 )
coef0 <- c(cdt1, cdt, ca)

#   Check of gradient  
check.derivatives(coef0, slikTP, gradlikTP, check_derivatives_tol = 1e-04, check_derivatives_print = 'all')

#   Estimation of preference  
valbl <- nloptr(coef0, eval_f = slikTP, eval_grad_f = gradlikTP,
                opts = list("algorithm" = "NLOPT_LD_LBFGS", "print_level" = 0,  "xtol_rel" = 1.0e-4,
                            "maxtime"=25000, "maxeval" = 2000))
bprefsp3 <- valbl$solution

# comupting the standard errors 
hesssp3 <- fdHess(bprefsp3, slikTP)[[3]]
vcovsp3 <- solve(hesssp3)
sdprefsp3 <- sqrt(abs(diag(vcovsp3)))
#  p values  
pvalsp3 <- pnorm(-abs(bprefsp3/sdprefsp3))*2

#  getting the deltas  
delta10sp3 <- 1/(1+exp(-cbind(rep(1,nobs), CFF) %*% bprefsp3[1:ndt]))
delta11sp3 <- (1-delta10sp3)/(1+exp(-cbind(rep(1,nobs), CFF) %*% bprefsp3[(ndt+1):(2*ndt)]))
delta01sp3 <- 1-delta10sp3 -delta11sp3
#  the matrix of Deltas for each individual
DTsp3 <- cbind(delta10sp3, delta11sp3, delta01sp3)


# prepare for the table
sp1 <- cbind.data.frame(varname=I(c( "delta_s_0", 
                           "delta_s_m0", 
                           "delta_sf_0",
                           "delta_sf_m0", 
                           "intercept",
                           "ability_m0", 
                           "female",
                           "age",
                           "repetition",
                           "educ_father",
                           "educ_mother")),bpref, sdpref)

sp2 <- cbind.data.frame(varname=I(c( "delta_s_0", 
                           "delta_s_m0",
                           "delta_s_in_forta",
                           "delta_sf_0",
                           "delta_sf_m0",
                           "delta_sf_in_forta",
                           "intercept",
                           "ability_m0",
                           "in_forta",
                           "female",
                           "age",
                           "repetition",
                           "educ_father",
                           "educ_mother")),bprefsp2, sdprefsp2)

sp3 <- cbind.data.frame(varname=I(c( "delta_s_0",  
                            "delta_s_in_forta",
                            "delta_sf_0", 
                            "delta_sf_in_forta",
                            "intercept",
                            "ability_m0",
                            "in_forta",
                            "female",
                            "age",
                            "repetition",
                            "educ_father",
                            "educ_mother")),bprefsp3, sdprefsp3)

detach(package:dplyr)
library(plyr) 
table_S_vii<-join(join(sp2, sp1,   by="varname" ),sp3, by="varname")
table_S_vii <- table_S_vii[,c(1,4,5,2,3,6,7)]
colnames(table_S_vii) <- c("varname","coef sp1","s.d. sp1", "coef sp2", "s.d. sp2", "coef sp3","s.d. sp3")

#================================= Table S.vii output ====================================================
print(table_S_vii, digits=2)




###########################################################################################################
###         Appendix S figures                                                                           ##
###########################################################################################################

#============================ Figure S.i===================================================================
par(mfrow=c(2,2))
plot(density(m1[Number==48], na.rm=TRUE),main="Sobral m1 density", font.main=2)
plot(density(m1[Number==49], na.rm=TRUE),main="Fortaleza m1 density", font.main=2) 
plot(density(dat$FS_SSE[Number==48], na.rm=TRUE),main="Sobral m2 density", font.main=2) 
plot(density(dat$FS_SSE[Number==49], na.rm=TRUE),main="Fortaleza m2 density", font.main=2) 


#========================= Figure S.ii  ====================================================================
par(mfrow=c(1,1))
plot(epsdat, m1, col="white", main = "Ability and first stage grades", ylab="First stage grades", xlab="Ability(normalized Enem)")
lines(lowess(epsdat, m1, f =0.15, iter=5), lwd=2, col='blue')


#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#====================================== figure S.iii. ============================================================================

par(mfrow=c(1,1))
# get the percentiles of the final grades 
qtfs_sob_f  <-quantile(FS_FSE[Number==48], probs=seq(0,1, by=0.02))
qtfs_fort_f <-quantile(FS_FSE[Number==49], probs=seq(0,1, by=0.02))

# utility difference for all students
odd1_f<-rep(0,length(qtfs_sob_f)-1)
odd2_f<-rep(0, length(qtfs_sob_f)-1)
for (i in 1:length(qtfs_sob_f)-1)
{
  odd1_f[i]<-mean((p1s_table[Number==48&FS_FSE>=qtfs_sob_f[i] & FS_FSE<qtfs_sob_f[i+1]]/p1f_table[Number==48&FS_FSE>=qtfs_sob_f[i] & FS_FSE<qtfs_sob_f[i+1]]))/mean((p1s_table[Number==49&FS_FSE>=qtfs_fort_f[i] & FS_FSE<qtfs_fort_f[i+1]]/p1f_table[Number==49&FS_FSE>=qtfs_fort_f[i] & FS_FSE<qtfs_fort_f[i+1]]))
  odd2_f[i]<-mean((p2s_table[Number==48&FS_FSE>=qtfs_sob_f[i] & FS_FSE<qtfs_sob_f[i+1]]/p2f_table[Number==48&FS_FSE>=qtfs_sob_f[i] & FS_FSE<qtfs_sob_f[i+1]]))/mean((p2s_table[Number==49&FS_FSE>=qtfs_fort_f[i] & FS_FSE<qtfs_fort_f[i+1]]/p2f_table[Number==49&FS_FSE>=qtfs_fort_f[i] & FS_FSE<qtfs_fort_f[i+1]]))
}

odd1_f
odd2_f
x_axis_odd <- seq(0.02,1, by=0.02)

plot(x_axis_odd,odd1_f,  col="red", pch=8, cex=0.5, ann=FALSE)
points(x_axis_odd, odd2_f, col='blue', pch=2, cex=0.5)
abline(h=1, col="black")
title(main="The Odds ratio plot of simulated success probabilities", font.main=2)
title(ylab="Odds Ratio")
title(xlab="Percentile")






